=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2005-12-08 07:13:08 +0000
+++ bzrlib/branch.py	2005-12-15 21:11:09 +0000
@@ -562,25 +562,23 @@
         if init:
             self._make_control()
         self._check_format(relax_version_check)
+        self._find_modes()
 
         def get_store(name, compressed=True, prefixed=False):
-            # FIXME: This approach of assuming stores are all entirely compressed
-            # or entirely uncompressed is tidy, but breaks upgrade from 
-            # some existing branches where there's a mixture; we probably 
-            # still want the option to look for both.
             relpath = self._rel_controlfilename(unicode(name))
             store = TextStore(self._transport.clone(relpath),
+                              dir_mode=self._dir_mode,
+                              file_mode=self._file_mode,
                               prefixed=prefixed,
                               compressed=compressed)
-            #if self._transport.should_cache():
-            #    cache_path = os.path.join(self.cache_root, name)
-            #    os.mkdir(cache_path)
-            #    store = bzrlib.store.CachedStore(store, cache_path)
             return store
 
         def get_weave(name, prefixed=False):
             relpath = self._rel_controlfilename(unicode(name))
-            ws = WeaveStore(self._transport.clone(relpath), prefixed=prefixed)
+            ws = WeaveStore(self._transport.clone(relpath),
+                            prefixed=prefixed,
+                            dir_mode=self._dir_mode,
+                            file_mode=self._file_mode)
             if self._transport.should_cache():
                 ws.enable_cache = True
             return ws
@@ -753,7 +751,21 @@
                     f = codecs.getwriter('utf-8')(f, errors='replace')
             path = self._rel_controlfilename(path)
             ctrl_files.append((path, f))
-        self._transport.put_multi(ctrl_files)
+        self._transport.put_multi(ctrl_files, mode=self._file_mode)
+
+    def _find_modes(self, path=None):
+        """Determine the appropriate modes for files and directories."""
+        try:
+            if path is None:
+                path = self._rel_controlfilename('')
+            st = self._transport.stat(path)
+        except errors.TransportNotPossible:
+            self._dir_mode = 0755
+            self._file_mode = 0644
+        else:
+            self._dir_mode = st.st_mode & 07777
+            # Remove the sticky and execute bits for files
+            self._file_mode = self._dir_mode & ~07111
 
     def _make_control(self):
         from bzrlib.inventory import Inventory
@@ -771,7 +783,12 @@
         bzrlib.weavefile.write_weave_v5(Weave(), sio)
         empty_weave = sio.getvalue()
 
-        dirs = [[], 'revision-store', 'weaves']
+        cfn = self._rel_controlfilename
+        # Since we don't have a .bzr directory, inherit the
+        # mode from the root directory
+        self._find_modes(u'.')
+
+        dirs = ['', 'revision-store', 'weaves']
         files = [('README', 
             "This is a Bazaar-NG control directory.\n"
             "Do not change any files in this directory.\n"),
@@ -784,8 +801,7 @@
             ('inventory.weave', empty_weave),
             ('ancestry.weave', empty_weave)
         ]
-        cfn = self._rel_controlfilename
-        self._transport.mkdir_multi([cfn(d) for d in dirs])
+        self._transport.mkdir_multi([cfn(d) for d in dirs], mode=self._dir_mode)
         self.put_controlfiles(files)
         mutter('created control directory in ' + self._transport.base)
 

=== modified file 'bzrlib/store/__init__.py'
--- bzrlib/store/__init__.py	2005-11-30 00:25:50 +0000
+++ bzrlib/store/__init__.py	2005-12-15 21:11:09 +0000
@@ -144,6 +144,10 @@
 class TransportStore(Store):
     """A TransportStore is a Store superclass for Stores that use Transports."""
 
+    # Just some constants so that plugins/etc could override them if they want
+    _default_dir_mode = 0755
+    _default_file_mode = 0644
+
     def add(self, f, fileid, suffix=None):
         """Add contents of a file into the store.
 
@@ -220,7 +224,8 @@
                 pass
         raise KeyError(fileid)
 
-    def __init__(self, a_transport, prefixed=False, compressed=False):
+    def __init__(self, a_transport, prefixed=False, compressed=False,
+                 dir_mode=None, file_mode=None):
         assert isinstance(a_transport, transport.Transport)
         super(TransportStore, self).__init__()
         self._transport = a_transport
@@ -228,6 +233,18 @@
         self._compressed = compressed
         self._suffixes = set()
 
+        if dir_mode is None:
+            # This really should only happen in the test suite
+            self._dir_mode = self._default_dir_mode
+        else:
+            self._dir_mode = dir_mode
+
+        if file_mode is None:
+            # This really should only happen in the test suite
+            self._file_mode = self._default_file_mode
+        else:
+            self._file_mode = file_mode
+
     def _iter_files_recursive(self):
         """Iterate through the files in the transport."""
         for quoted_relpath in self._transport.iter_files_recursive():
@@ -303,37 +320,6 @@
     return bzrlib.store.text.TextStore(transport.memory.MemoryTransport())
         
 
-class CachedStore(Store):
-    """A store that caches data locally, to avoid repeated downloads.
-    The precacache method should be used to avoid server round-trips for
-    every piece of data.
-    """
-
-    def __init__(self, store, cache_dir):
-        super(CachedStore, self).__init__()
-        self.source_store = store
-        # This clones the source store type with a locally bound
-        # transport. FIXME: it assumes a constructor is == cloning.
-        # clonable store - it might be nicer to actually have a clone()
-        # or something. RBC 20051003
-        self.cache_store = store.__class__(LocalTransport(cache_dir))
-
-    def get(self, id):
-        mutter("Cache add %s", id)
-        if id not in self.cache_store:
-            self.cache_store.add(self.source_store.get(id), id)
-        return self.cache_store.get(id)
-
-    def has_id(self, fileid, suffix=None):
-        """See Store.has_id."""
-        if self.cache_store.has_id(fileid, suffix):
-            return True
-        if self.source_store.has_id(fileid, suffix):
-            # We could asynchronously copy at this time
-            return True
-        return False
-
-
 def copy_all(store_from, store_to):
     """Copy all ids from one store to another."""
     # TODO: Optional progress indicator

=== modified file 'bzrlib/store/text.py'
--- bzrlib/store/text.py	2005-11-10 02:11:13 +0000
+++ bzrlib/store/text.py	2005-12-15 21:11:09 +0000
@@ -65,15 +65,15 @@
 
     def _try_put(self, fn, f):
         try:
-            self._transport.put(fn, f)
+            self._transport.put(fn, f, mode=self._file_mode)
         except NoSuchFile:
             if not self._prefixed:
                 raise
             try:
-                self._transport.mkdir(os.path.dirname(fn))
+                self._transport.mkdir(os.path.dirname(fn), mode=self._dir_mode)
             except FileExists:
                 pass
-            self._transport.put(fn, f)
+            self._transport.put(fn, f, mode=self._file_mode)
 
     def _get(self, fn):
         if fn.endswith('.gz'):
@@ -95,15 +95,17 @@
             raise KeyError(fileid + '-' + str(suffix))
 
         try:
-            result = other._transport.copy_to([path], self._transport, pb=pb)
+            result = other._transport.copy_to([path], self._transport, pb=pb,
+                                              mode=self._file_mode)
         except NoSuchFile:
             if not self._prefixed:
                 raise
             try:
-                self._transport.mkdir(hash_prefix(fileid)[:-1])
+                self._transport.mkdir(hash_prefix(fileid)[:-1], mode=self._dir_mode)
             except FileExists:
                 pass
-            result = other._transport.copy_to([path], self._transport, pb=pb)
+            result = other._transport.copy_to([path], self._transport, pb=pb,
+                                              mode=self._file_mode)
 
         if result != 1:
             raise BzrError('Unable to copy file: %r' % (path,))

=== modified file 'bzrlib/store/weave.py'
--- bzrlib/store/weave.py	2005-12-08 07:35:29 +0000
+++ bzrlib/store/weave.py	2005-12-15 21:11:09 +0000
@@ -40,9 +40,11 @@
     """
     FILE_SUFFIX = '.weave'
 
-    def __init__(self, transport, prefixed=False, precious=False):
-        self._transport = transport
-        self._prefixed = prefixed
+    def __init__(self, transport, prefixed=False, precious=False,
+                 dir_mode=None, file_mode=None):
+        super(WeaveStore, self).__init__(transport,
+                dir_mode=dir_mode, file_mode=file_mode,
+                prefixed=prefixed, compressed=False)
         self._precious = precious
 
     def filename(self, file_id):
@@ -67,10 +69,10 @@
     def _put(self, file_id, f):
         if self._prefixed:
             try:
-                self._transport.mkdir(hash_prefix(file_id))
+                self._transport.mkdir(hash_prefix(file_id), mode=self._dir_mode)
             except FileExists:
                 pass
-        return self._transport.put(self.filename(file_id), f)
+        return self._transport.put(self.filename(file_id), f, mode=self._file_mode)
 
     def get_weave(self, file_id, transaction):
         weave = transaction.map.find_weave(file_id)

=== modified file 'bzrlib/tests/test_permissions.py'
--- bzrlib/tests/test_permissions.py	2005-12-15 20:03:03 +0000
+++ bzrlib/tests/test_permissions.py	2005-12-15 21:11:09 +0000
@@ -42,6 +42,8 @@
 
 def chmod_r(base, file_mode, dir_mode):
     """Recursively chmod from a base directory"""
+    assert os.path.isdir(base)
+    os.chmod(base, dir_mode)
     for root, dirs, files in os.walk(base):
         for d in dirs:
             p = os.path.join(root, d)
@@ -61,6 +63,9 @@
             p = os.path.join(root, f)
             check_mode(test, p, file_mode)
 
+def assertEqualMode(test, mode, mode_test):
+    test.assertEqual(mode, mode_test,
+                     'mode mismatch %o != %o' % (mode, mode_test))
 
 class TestPermissions(TestCaseInTempDir):
 
@@ -81,6 +86,8 @@
 
         b = Branch.open('.')
         t = b.working_tree()
+        assertEqualMode(self, 0755, b._dir_mode)
+        assertEqualMode(self, 0644, b._file_mode)
 
         # Modifying a file shouldn't break the permissions
         open('a', 'wb').write('foo2\n')
@@ -100,6 +107,8 @@
         check_mode_r(self, '.bzr', 0664, 0775)
         b = Branch.open('.')
         t = b.working_tree()
+        assertEqualMode(self, 0775, b._dir_mode)
+        assertEqualMode(self, 0664, b._file_mode)
 
         open('a', 'wb').write('foo3\n')
         t.commit('foo3')
@@ -110,6 +119,31 @@
         t.commit('new c')
         check_mode_r(self, '.bzr', 0664, 0775)
 
+        # Test the group sticky bit
+        del b, t
+        # Recursively update the modes of all files
+        chmod_r('.bzr', 0664, 02775)
+        check_mode_r(self, '.bzr', 0664, 02775)
+        b = Branch.open('.')
+        t = b.working_tree()
+        assertEqualMode(self, 02775, b._dir_mode)
+        assertEqualMode(self, 0664, b._file_mode)
+
+        open('a', 'wb').write('foo4\n')
+        t.commit('foo4')
+        check_mode_r(self, '.bzr', 0664, 02775)
+
+        open('d', 'wb').write('new d\n')
+        t.add('d')
+        t.commit('new d')
+        check_mode_r(self, '.bzr', 0664, 02775)
+
+try:
+    import paramiko
+except ImportError:
+    has_paramiko = False
+else:
+    has_paramiko = True
 
 class TestSftpPermissions(TestCaseWithSFTPServer):
 
@@ -119,10 +153,16 @@
         # Though it would be nice to test that SFTP to a server
         # which does support chmod has the right effect
 
+        if not has_paramiko:
+            raise TestSkipped('You must have paramiko installed to test sftp')
+
+        from bzrlib.transport.sftp import SFTPTransport
+
         # We don't actually use it directly, we just want to
         # keep the connection open, since StubSFTPServer only
-        # allows 1 connection. Also, this calls delayed_setup()
-        _transport = self.get_transport()
+        # allows 1 connection
+        self.delayed_setup()
+        _transport = SFTPTransport(self._sftp_url)
 
         os.mkdir('local')
         b_local = Branch.initialize(u'local')
@@ -138,10 +178,12 @@
 
         b_local = Branch.open(u'local')
         t_local = b_local.working_tree()
-        os.chdir('..')
+        assertEqualMode(self, 0755, b_local._dir_mode)
+        assertEqualMode(self, 0644, b_local._file_mode)
 
         os.mkdir('sftp')
-        sftp_url = self._sftp_url + '/sftp'
+        # Why does self._sftp_url end with a slash????
+        sftp_url = self._sftp_url + 'sftp'
         b_sftp = Branch.initialize(sftp_url)
 
         b_sftp.pull(b_local)
@@ -150,6 +192,8 @@
         check_mode_r(self, 'sftp/.bzr', 0644, 0755)
 
         b_sftp = Branch.open(sftp_url)
+        assertEqualMode(self, 0755, b_sftp._dir_mode)
+        assertEqualMode(self, 0644, b_sftp._file_mode)
 
         open('local/a', 'wb').write('foo2\n')
         t_local.commit('foo2')
@@ -157,7 +201,7 @@
         # The mode should be maintained after commit
         check_mode_r(self, 'sftp/.bzr', 0644, 0755)
 
-        open('b', 'wb').write('new b\n')
+        open('local/b', 'wb').write('new b\n')
         t_local.add('b')
         t_local.commit('new b')
         b_sftp.pull(b_local)
@@ -169,16 +213,17 @@
         check_mode_r(self, 'sftp/.bzr', 0664, 0775)
 
         b_sftp = Branch.open(sftp_url)
+        assertEqualMode(self, 0775, b_sftp._dir_mode)
+        assertEqualMode(self, 0664, b_sftp._file_mode)
 
-        open('a', 'wb').write('foo3\n')
+        open('local/a', 'wb').write('foo3\n')
         t_local.commit('foo3')
         b_sftp.pull(b_local)
         check_mode_r(self, 'sftp/.bzr', 0664, 0775)
 
-        open('c', 'wb').write('new c\n')
+        open('local/c', 'wb').write('new c\n')
         t_local.add('c')
         t_local.commit('new c')
         b_sftp.pull(b_local)
         check_mode_r(self, 'sftp/.bzr', 0664, 0775)
 
-

=== modified file 'bzrlib/tests/test_store.py'
--- bzrlib/tests/test_store.py	2005-12-01 21:31:42 +0000
+++ bzrlib/tests/test_store.py	2005-12-15 21:11:09 +0000
@@ -92,7 +92,8 @@
         self.assertEqual(store.total_size(), (2, 55))
         
     def test__relpath_suffixed(self):
-        my_store = TextStore(MockTransport(), True, compressed=True)
+        my_store = TextStore(MockTransport(),
+                             prefixed=True, compressed=True)
         my_store.register_suffix('dsc')
         self.assertEqual('45/foo.dsc', my_store._relpath('foo', ['dsc']))
 
@@ -357,7 +358,8 @@
                          my_store.get('missing', 'sig').read())
 
     def test___iter__no_suffix(self):
-        my_store = TextStore(MemoryTransport(), False, compressed=False)
+        my_store = TextStore(MemoryTransport(),
+                             prefixed=False, compressed=False)
         stream = StringIO("content")
         my_store.add(stream, "foo")
         self.assertEqual(set(['foo']),
@@ -382,7 +384,8 @@
 
     def test_copy_suffixes(self):
         from_store = self.get_populated_store()
-        to_store = TextStore(MemoryTransport(), True, compressed=True)
+        to_store = TextStore(MemoryTransport(),
+                             prefixed=True, compressed=True)
         to_store.register_suffix('sig')
         copy_all(from_store, to_store)
         self.assertEqual(1, len(to_store))

=== modified file 'bzrlib/tests/test_transport.py'
--- bzrlib/tests/test_transport.py	2005-12-15 20:05:02 +0000
+++ bzrlib/tests/test_transport.py	2005-12-15 21:11:09 +0000
@@ -17,6 +17,7 @@
 
 import os
 import sys
+import stat
 from cStringIO import StringIO
 
 from bzrlib.errors import (NoSuchFile, FileExists,
@@ -38,8 +39,7 @@
 if sys.platform != 'win32':
     def check_mode(test, path, mode):
         """Check that a particular path has the correct mode."""
-        # We are ignoring stuff like group sticky bits
-        actual_mode = os.stat(path).st_mode & 0777
+        actual_mode = stat.S_IMODE(os.stat(path).st_mode)
         test.assertEqual(mode, actual_mode,
             'mode of %r incorrect (%o != %o)' % (path, mode, actual_mode))
 else:

=== modified file 'bzrlib/transport/memory.py'
--- bzrlib/transport/memory.py	2005-12-01 21:31:42 +0000
+++ bzrlib/transport/memory.py	2005-12-15 21:11:09 +0000
@@ -67,12 +67,12 @@
             raise NoSuchFile(relpath)
         return StringIO(self._files[relpath])
 
-    def put(self, relpath, f):
+    def put(self, relpath, f, mode=None):
         """See Transport.put()."""
         self._check_parent(relpath)
         self._files[relpath] = f.read()
 
-    def mkdir(self, relpath):
+    def mkdir(self, relpath, mode=None):
         """See Transport.mkdir()."""
         self._check_parent(relpath)
         if relpath in self._dirs:

=== modified file 'bzrlib/transport/sftp.py'
--- bzrlib/transport/sftp.py	2005-12-15 20:03:03 +0000
+++ bzrlib/transport/sftp.py	2005-12-15 21:11:09 +0000
@@ -434,7 +434,7 @@
             if mode is not None:
                 self._sftp.chmod(path, mode=mode)
         except (paramiko.SSHException, IOError), e:
-            self._translate_io_exception(e, relpath, ': unable to mkdir',
+            self._translate_io_exception(e, path, ': unable to mkdir',
                 failure_exc=FileExists)
 
     def _translate_io_exception(self, e, path, more_info='', failure_exc=NoSuchFile):

